import { world } from "@minecraft/server";
import { ActionFormData, ModalFormData } from '@minecraft/server-ui';
import { v3_to_array, arr_to_v3, num_to_bool } from '../func';
function mult_run_command(exe, commands, times = 1) {
    while (times >= 1) {
        for (let c of commands) {
            exe.runCommandAsync(c).catch(e => console.warn(e));
        }
        times = times - 1;
    }
}
function get_score(sb_id, name) {
    let p = NaN;
    world.scoreboard.getObjective(sb_id).getScores().forEach(f => {
        if (f.participant.displayName == name) {
            p = f.score;
        }
    });
    return p;
}
export function player_property_check() {
    var _a, _b;
    let max_health_af = (_a = world.scoreboard.getObjective('global')) === null || _a === void 0 ? void 0 : _a.getScore('max_health');
    let maxmagic_af = (_b = world.scoreboard.getObjective('global')) === null || _b === void 0 ? void 0 : _b.getScore('maxmagic');
    world.getAllPlayers().forEach(p => {
        p.getTags().forEach(t => {
            var _a;
            if (t.indexOf('hpl') != -1) {
                p.removeTag(t);
            }
            let tag_name = 'hpl' + String(Math.round((max_health_af - 8) / 2));
            p.addTag(tag_name);
            (_a = world.scoreboard.getObjective('maxmagic')) === null || _a === void 0 ? void 0 : _a.setScore(p, maxmagic_af);
        });
    });
}
export class crystal_manage_book {
    constructor() { }
    static crystal_manage_book_index(player, block_l) {
        let index = new ActionFormData();
        index.title('text.dec:crystal_manage_book_title.name');
        index.body('text.dec:crystal_manage_book_index_body.name');
        index.button('text.dec:crystal_manage_book_index_create.name');
        index.button('text.dec:crystal_manage_book_index_manage.name');
        index.button('text.dec:crystal_manage_book_index_set.name');
        index.show(player).then(t => {
            if (t.selection == 0) {
                this.crystal_manage_book_create(player, block_l);
            }
            else if (t.selection == 1) {
                this.crystal_manage_book_manage(player);
            }
            else if (t.selection == 2) {
                this.crystal_manage_book_set(player);
            }
        });
    }
    static crystal_manage_book_create(player, block_l) {
        let index = new ModalFormData();
        index.title('text.dec:crystal_manage_book_title.name');
        index.dropdown('text.dec:crystal_manage_book_create_body.name', [
            'text.dec:crystal_manage_book_create_crystal.name',
            'text.dec:crystal_manage_book_create_tp_point.name',
            'text.dec:crystal_manage_book_create_spawn_point.name'
        ], 0);
        index.textField('text.dec:crystal_manage_book_create_entername.name', '', '');
        index.textField('text.dec:crystal_manage_book_create_spawn_point_entity.name', '', '');
        index.textField('text.dec:crystal_manage_book_create_spawn_point_wait_tick.name', '', '200');
        index.show(player).then(t => {
            if (t.formValues[0] == 0) {
                world.getDimension('overworld').runCommandAsync('summon dec:conquer_crystal ' + String(block_l.x) + ' ' + String(block_l.y + 1) + ' ' + String(block_l.z) + ' 0 0 minecraft:entity_spawned ' + String(t.formValues[1]));
            }
            else if (t.formValues[0] == 1) {
                let x = block_l.x;
                let y = block_l.y + 1;
                let z = block_l.z;
                mult_run_command(world.getDimension('overworld'), [
                    'scoreboard players set tpp_' + String(t.formValues[1]) + '_x global ' + String(x),
                    'scoreboard players set tpp_' + String(t.formValues[1]) + '_y global ' + String(y),
                    'scoreboard players set tpp_' + String(t.formValues[1]) + '_z global ' + String(z),
                ]);
            }
            else if (t.formValues[0] == 2) {
                try {
                    let loc = block_l;
                    loc.y += 1;
                    let point_data = {
                        'location': v3_to_array(loc),
                        'dimension': player.dimension.id,
                        'entity': t.formValues[2],
                        'wait_ticks': Number(t.formValues[3])
                    };
                    if (world.getDynamicPropertyIds().indexOf('spawn_points') != -1) {
                        let spawn_points = JSON.parse(world.getDynamicProperty('spawn_points'));
                        spawn_points[String(t.formValues[1])] = point_data;
                        world.setDynamicProperty('spawn_points', JSON.stringify(spawn_points));
                    }
                    else {
                        let spawn_points = {};
                        spawn_points[String(t.formValues[1])] = point_data;
                        world.setDynamicProperty('spawn_points', JSON.stringify(spawn_points));
                    }
                }
                catch (error) {
                    player.runCommandAsync('titleraw @s actionbar { "rawtext" : [ { "translate" : "text.dec:crystal_manage_book_create_spawn_point_fail.name" } ] }');
                    t.canceled;
                }
            }
        });
    }
    static crystal_manage_book_manage(player) {
        let index = new ModalFormData();
        index.title('text.dec:crystal_manage_book_title.name');
        let manage_body = new Array;
        for (let c of world.getDimension('overworld').getEntities({
            type: 'dec:conquer_crystal'
        })) {
            manage_body.push('Crystal ' + c.nameTag);
        }
        let tpp_coor = {};
        world.scoreboard.getObjective('global').getScores().forEach(o => {
            if (o.participant.displayName.slice(0, 4) == 'tpp_') {
                let name = o.participant.displayName.slice(4, -2);
                let c = o.participant.displayName.slice(-1);
                let num = o.score;
                if (name in tpp_coor == false) {
                    tpp_coor[name] = {};
                    manage_body.push('Tppoint ' + name);
                }
                tpp_coor[name][c] = num;
            }
        });
        if (world.getDynamicPropertyIds().indexOf('spawn_points') != -1) {
            let spawn_points = JSON.parse(world.getDynamicProperty('spawn_points'));
            for (let k of Object.keys(spawn_points)) {
                manage_body.push('Spawnpoint ' + k);
            }
        }
        index.dropdown('text.dec:crystal_manage_book_manage_body.name', manage_body, 0);
        index.dropdown('text.dec:crystal_manage_book_manage_act.name', [
            'text.dec:crystal_manage_book_manage_tp.name',
            'text.dec:crystal_manage_book_manage_modify.name',
            'text.dec:crystal_manage_book_manage_delete.name' //2
        ], 0);
        index.textField('text.dec:crystal_manage_book_rename.name', '', '');
        index.textField('text.dec:crystal_manage_book_rename_spawn_point_entity.name', '', '');
        index.textField('text.dec:crystal_manage_book_rename_spawn_point_wait_ticks.name', '', '');
        index.show(player).then(t => {
            let ob_type = '';
            let choose_ob = manage_body[t.formValues[0]];
            if (choose_ob.slice(0, 10) == 'Spawnpoint') {
                ob_type = 'Spawnpoint';
            }
            else if (choose_ob.slice(0, 7) == 'Tppoint' || choose_ob.slice(0, 7) == 'Crystal') {
                ob_type = choose_ob.slice(0, 7);
            }
            let ob_name = manage_body[t.formValues[0]].slice(8);
            if (ob_type == 'Crystal') {
                if (Number(t.formValues[1] == 0)) {
                    player.runCommandAsync('tp @s @e[c=1,type=dec:conquer_crystal,name=\"' + ob_name + '\"]');
                }
                else if (Number(t.formValues[1] == 1)) {
                    mult_run_command(player, [
                        'execute at @e[c=1,type=dec:conquer_crystal,name=\"' + ob_name + '\"] run summon dec:conquer_crystal ~~~ 0 0 minecraft:entity_spawned ' + String(t.formValues[2]),
                        'event entity @e[c=1,type=dec:conquer_crystal,name=\"' + ob_name + '\"] minecraft:despawn'
                    ]);
                }
                else if (Number(t.formValues[1] == 2)) {
                    player.runCommandAsync('event entity @e[c=1,type=dec:conquer_crystal,name=\"' + ob_name + '\"] minecraft:despawn');
                }
            }
            else if (ob_type == 'Tppoint') {
                let x = tpp_coor[ob_name]['x'];
                let y = tpp_coor[ob_name]['y'];
                let z = tpp_coor[ob_name]['z'];
                if (Number(t.formValues[1] == 0)) {
                    player.runCommandAsync('tp @s ' + String(x) + ' ' + String(y) + " " + String(z));
                }
                else if (Number(t.formValues[1] == 1)) {
                    mult_run_command(player, [
                        'scoreboard players set tpp_' + String(t.formValues[2]) + '_x global ' + String(x),
                        'scoreboard players set tpp_' + String(t.formValues[2]) + '_y global ' + String(y),
                        'scoreboard players set tpp_' + String(t.formValues[2]) + '_z global ' + String(z),
                        'scoreboard players reset tpp_' + ob_name + '_x global',
                        'scoreboard players reset tpp_' + ob_name + '_y global',
                        'scoreboard players reset tpp_' + ob_name + '_z global'
                    ]);
                }
                else if (Number(t.formValues[1] == 2)) {
                    mult_run_command(player, [
                        'scoreboard players reset tpp_' + ob_name + '_x global',
                        'scoreboard players reset tpp_' + ob_name + '_y global',
                        'scoreboard players reset tpp_' + ob_name + '_z global'
                    ]);
                }
            }
            else if (ob_type == 'Spawnpoint') {
                let spawn_point_name = choose_ob.slice(11);
                let spawn_point = JSON.parse(world.getDynamicProperty('spawn_points'))[spawn_point_name];
                if (Number(t.formValues[1] == 0)) {
                    player.teleport(arr_to_v3(spawn_point['location']));
                }
                else if (Number(t.formValues[1] == 1)) {
                    //Modify
                    let loc = spawn_point['location'];
                    let entity = spawn_point['entity'];
                    let wait_ticks = spawn_point['wait_ticks'];
                    if (t.formValues[3] != '') {
                        entity = t.formValues[3];
                    }
                    if (t.formValues[4] != '') {
                        try {
                            wait_ticks = Number(t.formValues[4]);
                        }
                        catch (error) {
                            wait_ticks = spawn_point['wait_ticks'];
                            player.runCommandAsync('titleraw @s actionbar { "rawtext" : [ { "translate" : "text.dec:crystal_manage_book_manage_spawn_point_ticks_fail.name" } ] }');
                        }
                    }
                    let point_data = {
                        'location': loc,
                        'dimension': spawn_point['dimension'],
                        'entity': entity,
                        'wait_ticks': wait_ticks
                    };
                    let new_name = String(t.formValues[2]);
                    let new_spawn_points = JSON.parse(world.getDynamicProperty('spawn_points'));
                    if (new_name != '' && new_name != spawn_point_name) {
                        delete new_spawn_points[spawn_point_name];
                    }
                    new_spawn_points[new_name] = point_data;
                    world.setDynamicProperty('spawn_points', JSON.stringify(new_spawn_points));
                }
                else if (Number(t.formValues[1] == 2)) {
                    let new_spawn_points = JSON.parse(world.getDynamicProperty('spawn_points'));
                    delete new_spawn_points[spawn_point_name];
                    world.setDynamicProperty('spawn_points', JSON.stringify(new_spawn_points));
                }
            }
        });
    }
    static crystal_manage_book_set(player) {
        function check_init(value, init_value) {
            if (value < 0 || isNaN(value)) {
                value = init_value;
            }
            return value;
        }
        let index = new ModalFormData();
        index.title('text.dec:crystal_manage_book_title.name');
        let die_deduct = get_score('global', 'die_deduct');
        let max_health = get_score('global', 'max_health');
        let maxmagic = get_score('global', 'maxmagic');
        let random_equipment = get_score('global', 'random_equipment');
        die_deduct = check_init(die_deduct, 5);
        max_health = check_init(max_health, 10);
        maxmagic = check_init(maxmagic, 20);
        random_equipment = check_init(random_equipment, 0);
        index.slider('Die Deduct', 0, 100, 1, die_deduct); //text.dec:crystal_manage_book_manage_die_deduct.name
        index.slider('Max Health', 10, 24, 2, max_health); //No text temporary
        index.slider('Max Magic', 20, 60, 2, maxmagic); //No text temporary
        index.toggle('text.dec:crystal_manage_book_manage_random_equipment.name', num_to_bool(random_equipment));
        index.show(player).then(t => {
            var _a, _b, _c, _d;
            let max_health_af = t.formValues[1];
            let maxmagic_af = t.formValues[2];
            (_a = world.scoreboard.getObjective('global')) === null || _a === void 0 ? void 0 : _a.setScore('die_deduct', t.formValues[0]);
            (_b = world.scoreboard.getObjective('global')) === null || _b === void 0 ? void 0 : _b.setScore('max_health', max_health_af);
            (_c = world.scoreboard.getObjective('global')) === null || _c === void 0 ? void 0 : _c.setScore('maxmagic', maxmagic_af);
            (_d = world.scoreboard.getObjective('global')) === null || _d === void 0 ? void 0 : _d.setScore('random_equipment', t.formValues[3]);
            player_property_check();
        });
    }
}
//# sourceMappingURL=crystal_manage_book.js.map