import { crystal_manage_book, player_property_check } from "./items/crystal_manage_book"
import { crystal_perspective_book } from "./items/crystal_perspective_book"
import { crystal_teleport_book } from "./items/crystal_teleport_book"
import { world, Player, Entity, Dimension, system, Vector3, ScoreboardObjective, EntityQueryOptions, GameMode, ScoreboardIdentity } from '@minecraft/server';
import { arr_to_v3 } from "./func";

function find_max(target_arr: Array<number>) {
    let max = 0
    for (let n of target_arr) {
        if (n > max) {
            max = n
        }
    }
    return max
}

function str_repeat(str: String, times: number) {
    let s = ''
    let i = 0
    while (i < times) {
        s += str
        i++
    }
    return s
}

class player_score {
    name: string
    team: string
    kill: number
    die: number
    occupy: number
    total_score: number
    constructor(player: Player) {
        this.name = player.nameTag
        this.team = ''
        if (if_in('team_red', player.getTags())) {
            this.team = '§c§l'
        } else if (if_in('team_blue', player.getTags())) {
            this.team = '§b§l'
        }
        this.kill = get_score('kill_board', player.nameTag)
        if (isNaN(this.kill)) {
            this.kill = 0
        }
        this.die = get_score('die_board', player.nameTag)
        if (isNaN(this.die)) {
            this.die = 0
        }
        this.occupy = get_score('occupy_board', player.nameTag)
        if (isNaN(this.occupy)) {
            this.occupy = 0
        }
        this.total_score = 15 * this.kill + this.occupy - 3 * this.die
    }
}

function kill_die_occupy_list() {
    let txt = ''
    let player_score_list = new Array<player_score>

    let player_kill_list = new Array<number>
    let player_die_list = new Array<number>
    let player_occupy_list = new Array<number>
    let player_total_score_list = new Array<number>
    world.getAllPlayers().forEach(p => {
        if (if_in('team_blue', p.getTags()) || if_in('team_red', p.getTags())) {
            let p_s = new player_score(p)
            player_score_list.push(p_s)
            player_kill_list.push(p_s.kill)
            player_die_list.push(p_s.die)
            player_occupy_list.push(p_s.occupy)
            player_total_score_list.push(p_s.total_score)
        }
    })

    let kill_max = find_max(player_kill_list)
    let die_max = find_max(player_die_list)
    let occupy_max = find_max(player_occupy_list)
    let total_score_max = find_max(player_total_score_list)

    let kill_max_txt = ''
    let die_max_txt = ''
    let occupy_max_txt = ''
    let total_score_max_txt = ''

    for (let p_s of player_score_list) {
        txt = txt + p_s.team + p_s.name + '§r: ' + String(p_s.kill) + '|' + String(p_s.die) + '|' + String(p_s.occupy) + '\n'
        if (p_s.kill == kill_max) {
            kill_max_txt = kill_max_txt + ',' + p_s.team + p_s.name + '§r'
        }
        if (p_s.die == die_max) {
            die_max_txt = die_max_txt + ',' + p_s.team + p_s.name + '§r'
        }
        if (p_s.occupy == occupy_max) {
            occupy_max_txt = occupy_max_txt + ',' + p_s.team + p_s.name + '§r'
        }
        if (p_s.total_score == total_score_max) {
            total_score_max_txt = total_score_max_txt + ',' + p_s.team + p_s.name + '§r'
        }
    }
    kill_max_txt = kill_max_txt.slice(1)
    die_max_txt = die_max_txt.slice(1)
    occupy_max_txt = occupy_max_txt.slice(1)
    total_score_max_txt = total_score_max_txt.slice(1)
    mult_run_command(world.getDimension('overworld'), [
        'tellraw @a { "rawtext" : [ { "translate" : "text.dec:end_score_list.name" } ] }',
        'tellraw @a { "rawtext" : [ { "text": "' + txt + '" } ] }',
        'tellraw @a { "rawtext" : [ { "translate" : "text.dec:end_score_kill_max.name" } ] }',
        'tellraw @a { "rawtext" : [ { "text": "' + kill_max_txt + '" } ] }',
        'tellraw @a { "rawtext" : [ { "translate" : "text.dec:end_score_die_max.name" } ] }',
        'tellraw @a { "rawtext" : [ { "text": "' + die_max_txt + '" } ] }',
        'tellraw @a { "rawtext" : [ { "translate" : "text.dec:end_score_occupy_max.name" } ] }',
        'tellraw @a { "rawtext" : [ { "text": "' + occupy_max_txt + '" } ] }',
        'tellraw @a { "rawtext" : [ { "translate" : "text.dec:end_score_total_score_max.name" } ] }',
        'tellraw @a { "rawtext" : [ { "text": "' + total_score_max_txt + '" } ] }'
    ])
}

function get_score(sb_id: string, name: string) {
    let p = NaN
    let sbs_raw = world.scoreboard.getObjectives()
    let sbs = new Array<string>
    sbs_raw.forEach(sb => {
        sbs.push(sb.id)
    })
    if (if_in(sb_id, sbs)) {
        (<ScoreboardObjective>world.scoreboard.getObjective(sb_id)).getScores().forEach(f => {
            if (f.participant.displayName == name) {
                p = f.score
            }
        })
    }
    return p
}
function if_in(ele: any, lis: Array<any>) {
    let test = false
    for (let l of lis) {
        if (l == ele) {
            test = true
            break
        }
    }
    return test
}
function mult_run_command(exe: Player | Entity | Dimension, commands: Array<string>, times: number = 1) {
    while (times >= 1) {
        for (let c of commands) {
            exe.runCommandAsync(c).catch(e => console.warn(e)).catch(r => {
                exe.runCommandAsync("say " + String(r))
            })
        }
        times = times - 1
    }
}

try {
    world.scoreboard.addObjective('team_score', '§lTeamScore')
    world.scoreboard.addObjective('die_board', 'die_board')
    world.scoreboard.addObjective('kill_board', 'kill_board')
    world.scoreboard.addObjective('occupy_board', 'occupy_board')
} catch {

}
mult_run_command(world.getDimension('overworld'), [
    'scoreboard players add §b§lBlueTeam team_score 0',
    'scoreboard players add §c§lRedTeam team_score 0',
    'scoreboard objectives setdisplay sidebar team_score',

    'scoreboard players add score_judge_ticks global 0'
])

let die_deduct = 5
world.getDimension('overworld').runCommandAsync('say reload complete')

world.afterEvents.itemUse.subscribe(e => {
    let location_cor: Vector3 = { x: Math.floor(e.source.location.x), y: Math.floor(e.source.location.y) - 1, z: Math.floor(e.source.location.z) }
    if (e.itemStack.typeId == 'dec:crystal_manage_book') {
        crystal_manage_book.crystal_manage_book_index((<Player>e.source), location_cor)//e.block.location
    } else if (e.itemStack.typeId == 'dec:crystal_perspective_book') {
        if (e.source.isSneaking) {
            crystal_perspective_book.crystal_perspective_book_index((<Player>e.source))
        } else {
            crystal_perspective_book.crystal_perspective_book_tell((<Player>e.source))
        }
    } else if (e.itemStack.typeId == 'dec:crystal_teleport_book') {
        crystal_teleport_book.crystal_teleport_book_index((<Player>e.source))
    }
})

world.beforeEvents.entityRemove.subscribe(e => {
    if (e.removedEntity.typeId != 'minecraft:player') {
        e.removedEntity.getTags().forEach(t => {
            if (t.slice(0, 11) == 'spawnpoint_') {
                let spawn_point_name = t.slice(11)
                let spawn_points = JSON.parse(<string>world.getDynamicProperty('spawn_points'))
                if (Object.keys(spawn_points).indexOf(spawn_point_name) != -1) {
                    spawn_point_run(spawn_point_name, spawn_points[spawn_point_name])
                }
            }
        })
    }
})

world.afterEvents.entityDie.subscribe(e => {
    if (e.deadEntity.typeId == 'minecraft:player') {
        die_deduct = get_score('global', 'die_deduct')
        if (isNaN(die_deduct)) {
            die_deduct = 5
        }
        world.scoreboard.getObjective('global')?.setScore('die_deduct', die_deduct)
        if (if_in('gamer', e.deadEntity.getTags())) {
            e.deadEntity.runCommandAsync('scoreboard players add @s die_board 1')
            if (e.damageSource.damagingEntity != undefined && e.damageSource.damagingEntity.typeId == 'minecraft:player') {
                e.damageSource.damagingEntity.runCommandAsync('scoreboard players add @s kill_board 1')
            }
            if (if_in('team_blue', e.deadEntity.getTags())) {
                mult_run_command(e.deadEntity, [
                    'scoreboard players remove §b§lBlueTeam team_score ' + String(die_deduct),
                    'clear @s',
                    'tag @s remove gamer'
                ])
            } else if (if_in('team_red', e.deadEntity.getTags())) {
                mult_run_command(e.deadEntity, [
                    'scoreboard players remove §c§lRedTeam team_score ' + String(die_deduct),
                    'clear @s',
                    'tag @s remove gamer'
                ])
            }
        }
    }
})

const process_bar = 20
const process_len = 100 / process_bar

system.afterEvents.scriptEventReceive.subscribe(e => {
    if (e.id == 'dec:conquer_crystal') {
        let cc = <Entity>e.sourceEntity
        if (e.message == 'occupy_statistic') {
            const occupyStatisticSetOptions: EntityQueryOptions = {
                location: e.sourceEntity?.location,
                type: 'minecraft:player',
                tags: ['gamer'],
                maxDistance: 8,
                excludeGameModes: [GameMode.creative, GameMode.spectator]
            }
            const nearby_players = <Array<Player>>e.sourceEntity?.dimension.getEntities(occupyStatisticSetOptions)
            let occ_weight = 0
            let d_score = 0
            let cur_score = <number>world.scoreboard.getObjective('global')?.getScore(<ScoreboardIdentity>cc.scoreboardIdentity)
            if (nearby_players.length > 0) {
                //Have players nearby
                nearby_players.forEach(p => {
                    let d_sq = Math.pow(p.location.x - cc.location.x, 2) + Math.pow(p.location.y - cc.location.y, 2) + Math.pow(p.location.z - cc.location.z, 2)
                    let d_occ_weight = -0.125 * d_sq + 10
                    if (if_in('team_blue', p.getTags())) {
                        occ_weight += d_occ_weight
                    } else {
                        occ_weight -= d_occ_weight
                    }
                })
                if (Math.abs(occ_weight) > 25) {
                    d_score = Math.sign(occ_weight) * 25
                } else {
                    d_score = occ_weight
                }
                if (occ_weight > 0 && cur_score != 100) {
                    //Blue occupying
                    mult_run_command(cc, [
                        //'titleraw @e[type=player,r=8] actionbar { "rawtext" : [ { "translate" : "text.dec:conquer_crystal_becoming_blue.name" } ] }',
                        'scoreboard players add @a[r=8,tag=team_blue] occupy_board 1'
                    ])
                    let p_location = <Vector3>e.sourceEntity?.location
                    p_location.y += 1.3
                    world.getDimension('overworld').spawnParticle('dec:occupying_team_blue_particle', p_location)
                } else if (occ_weight < 0 && cur_score != -100) {
                    //Red occupying
                    mult_run_command(cc, [
                        //'titleraw @e[type=player,r=8] actionbar { "rawtext" : [ { "translate" : "text.dec:conquer_crystal_becoming_red.name" } ] }',
                        'scoreboard players add @a[r=8,tag=team_red] occupy_board 1'
                    ])
                    let p_location = <Vector3>e.sourceEntity?.location
                    p_location.y += 1.3
                    world.getDimension('overworld').spawnParticle('dec:occupying_team_red_particle', p_location)
                }
            } else {
                if (cur_score <= -70 && cur_score > -100) {
                    d_score = -2
                } else if (cur_score >= 70 && cur_score < 100) {
                    d_score = 2//This could be set later
                } else if (cur_score > -70 && cur_score < 0) {
                    d_score = 2
                } else if (cur_score < 70 && cur_score > 0) {
                    d_score = -2
                }
            }
            let after_score = cur_score + d_score
            if (after_score > 100) {
                after_score = 100
            } else if (after_score < -100) {
                after_score = -100
            }
            if (cur_score > -100 && after_score == -100) {
                //Be occupied by red
                mult_run_command(cc, [
                    'tellraw @a { "rawtext" : [ { "translate" : "text.dec:conquer_crystal_become_red.name","with": { "rawtext": [ { "selector": "@s" } ] } } ] }',
                    'event entity @s minecraft:red'
                ])
            } else if (cur_score < 100 && after_score == 100) {
                //Be occupied by blue
                mult_run_command(cc, [
                    'tellraw @a { "rawtext" : [ { "translate" : "text.dec:conquer_crystal_become_blue.name","with": { "rawtext": [ { "selector": "@s" } ] } } ] }',
                    'event entity @s minecraft:blue'
                ])
            }
            if (cur_score <= -70 && after_score > -70) {
                //Red losing
                mult_run_command(cc, [
                    'tellraw @a[tag=team_red] { "rawtext" : [ { "translate" : "text.dec:conquer_crystal_losing_red.name","with": { "rawtext": [ { "selector": "@s" } ] } } ] }',
                    'event entity @s minecraft:light_red'
                ])
            } else if (cur_score >= 70 && after_score < 70) {
                //Blue losing
                mult_run_command(cc, [
                    'tellraw @a[tag=team_blue] { "rawtext" : [ { "translate" : "text.dec:conquer_crystal_losing_blue.name","with": { "rawtext": [ { "selector": "@s" } ] } } ] }',
                    'event entity @s minecraft:light_blue'
                ])
            } else if (after_score <= -70 && after_score > -100) {
                mult_run_command(cc, [
                    'event entity @s minecraft:light_red'
                ])
            } else if (after_score >= 70 && after_score < 100) {
                mult_run_command(cc, [
                    'event entity @s minecraft:light_blue'
                ])
            }
            world.scoreboard.getObjective('global')?.setScore(<ScoreboardIdentity>cc.scoreboardIdentity, after_score)
            //Vision effect
            let crys_particle_location = <Vector3>e.sourceEntity?.location
            crys_particle_location.y += 3
            let ocp_msg = ''
            let colored_n = Math.floor(Math.abs(after_score) / process_len)
            let uncolored_n = process_bar - colored_n
            let score_msg = String(str_repeat('█', colored_n) + '§r§l' + str_repeat('█', uncolored_n))
            if (after_score == -100) {
                //Red
                world.getDimension('overworld').spawnParticle('dec:crystal_team_red_particle', crys_particle_location)
                ocp_msg = '§c§l' + score_msg
            } else if (after_score <= -70) {
                //Light Red
                world.getDimension('overworld').spawnParticle('dec:crystal_team_light_red_particle', crys_particle_location)
                ocp_msg = '§c§l' + score_msg
            } else if (after_score == 100) {
                //Blue
                world.getDimension('overworld').spawnParticle('dec:crystal_team_blue_particle', crys_particle_location)
                ocp_msg = '§b§l' + score_msg
            } else if (after_score >= 70) {
                //Light blue
                world.getDimension('overworld').spawnParticle('dec:crystal_team_light_blue_particle', crys_particle_location)
                ocp_msg = '§b§l' + score_msg
            } else {
                //White
                e.sourceEntity?.triggerEvent('minecraft:white')
                world.getDimension('overworld').spawnParticle('dec:crystal_team_white_particle', crys_particle_location)
                if (after_score > 0) {
                    ocp_msg = '§b§l' + score_msg
                } else if (after_score < 0) {
                    ocp_msg = '§c§l' + score_msg
                } else {
                    ocp_msg = '§r§l' + score_msg
                }
            }
            cc.runCommandAsync('title @a[r=8] actionbar ' + ocp_msg)
        }
    } else if (e.id == 'dec:conquer_start') {
        let max_score = 0
        try {
            max_score = Number(e.message)
        } catch {
            if (e.sourceEntity?.typeId == 'minecraft:player')
                mult_run_command(<Player>e.sourceEntity, [
                    'telleraw @s { "rawtext" : [ { "translate" : "text.dec:conquer_start_fail.name" } ] }'
                ])
        }
        if (max_score > 0) {
            spawn_point_clear()
            world.scoreboard.getObjective('team_score')?.setScore('§b§lBlueTeam', max_score)
            world.scoreboard.getObjective('team_score')?.setScore('§c§lRedTeam', max_score)
            e.sourceEntity?.dimension.runCommandAsync('function game/start')
            let spawn_points = JSON.parse(<string>world.getDynamicProperty('spawn_points'))
            Object.keys(spawn_points).forEach(k => {
                spawn_point_run(k, spawn_points[k])
            })
        } else {
            mult_run_command(<Player>e.sourceEntity, [
                'titleraw @s actionbar { "rawtext" : [ { "translate" : "text.dec:conquer_start_fail.name" } ] }'
            ])
        }
    } else if (e.id == 'dec:conquer_stop') {
        kill_die_occupy_list()
        mult_run_command(world.getDimension('overworld'), [
            'tellraw @a { "rawtext" : [ { "translate" : "text.dec:conquer_stop.name" } ] }',
            'function game/team_reset_all'
        ])
        game_end()
    }
})

const redSetOptions: EntityQueryOptions = {
    tags: ['team_red'],
    type: 'dec:conquer_crystal'
}
const blueSetOptions: EntityQueryOptions = {
    tags: ['team_blue'],
    type: 'dec:conquer_crystal'
}
const allCrysSetOptions: EntityQueryOptions = {
    type: 'dec:conquer_crystal'
}
function game_end() {
    world.getAllPlayers().forEach(p => {
        if (if_in('gamer', p.getTags())) {
            p.runCommandAsync('camera @s fade time 1 1 1 color 0 0 0')
            system.runTimeout(() => {
                p.teleport(world.getDefaultSpawnLocation())
            }, 25)
        }
    })
}
const tick = () => {
    //游戏在开始状态
    if (get_score('global', 'game_state') === 1) {

        //红队失败
        if (get_score('team_score', '§c§lRedTeam') <= 0) {
            kill_die_occupy_list()
            mult_run_command(world.getDimension('overworld'), [
                'tellraw @a { "rawtext" : [ { "translate" : "text.dec:game_red_fail.name","with": { "rawtext": [ { "selector": "@a[tag=team_blue]" } ] } } ] }',
                'function game/team_reset_all'
            ])
            game_end()
        }
        //蓝队失败
        if (get_score('team_score', '§b§lBlueTeam') <= 0) {
            kill_die_occupy_list()
            mult_run_command(world.getDimension('overworld'), [
                'tellraw @a { "rawtext" : [ { "translate" : "text.dec:game_blue_fail.name","with": { "rawtext": [ { "selector": "@a[tag=team_red]" } ] } } ] }',
                'function game/team_reset_all'
            ])
            game_end()
        }

        //Score removal
        let red_crys = world.getDimension('overworld').getEntities(redSetOptions)
        let blue_crys = world.getDimension('overworld').getEntities(blueSetOptions)
        let blue_n = blue_crys.length
        let red_n = red_crys.length
        let all_crys_n = world.getDimension('overworld').getEntities(allCrysSetOptions).length
        let judge_ticks = <number>world.scoreboard.getObjective('global')?.getScore('score_judge_ticks')
        if (judge_ticks <= 0 && blue_n + red_n > 0) {
            //Reset judge ticks
            judge_ticks = Math.abs(blue_n / (blue_n + red_n) - 0.5) * 80 + 20//can be set later
            if (all_crys_n == blue_n || all_crys_n == red_n) {
                judge_ticks = 10
            }
            //Score removal
            if (blue_n > red_n) {
                world.scoreboard.getObjective('team_score')?.addScore('§c§lRedTeam', -1)
            } else if (blue_n < red_n) {
                world.scoreboard.getObjective('team_score')?.addScore('§b§lBlueTeam', -1)
            }
        } else if (blue_n + red_n > 0) {
            judge_ticks -= 1
        }
        world.scoreboard.getObjective('global')?.setScore('score_judge_ticks', judge_ticks)
    }
}
system.runInterval(tick, 1)

function spawn_point_run(name: string, spawn_point: Record<string, any>) {
    if (get_score('global', 'game_state') == 1) {
        let location = arr_to_v3(spawn_point['location'])
        let entity_id = spawn_point['entity']
        let wait_ticks = Number(spawn_point['wait_ticks'])
        const spawnPointEntity: EntityQueryOptions = {
            tags: ['spawnpoint_' + name]
        }
        system.runTimeout(() => {
            if (world.getDimension('overworld').getEntities(spawnPointEntity).length == 0) {
                let entity = world.getDimension(spawn_point['dimension']).spawnEntity(entity_id, location)
                entity.addTag('spawnpoint_' + name)
            }
        }, wait_ticks)
    }
}

function spawn_point_clear() {
    let spawn_points = <Object>JSON.parse(<string>world.getDynamicProperty('spawn_points'))
    if (Object.keys(spawn_points).length > 0) {
        Object.keys(spawn_points).forEach(k => {
            const spawnPointEntity: EntityQueryOptions = {
                tags: ['spawnpoint_' + k]
            }
            world.getDimension('overworld').getEntities(spawnPointEntity).forEach(e=>{
                e.kill()
            })
        })
    }
}

function check() {
    if (get_score('global', 'game_state') === 1) {
        let spawn_points = JSON.parse(<string>world.getDynamicProperty('spawn_points'))
        Object.keys(spawn_points).forEach(k => {
            spawn_point_run(k, spawn_points[k])
        })
        player_property_check()
    }
}
system.runInterval(check, 400)